@@ -3,8 +3,9 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2006-2016, University of Amsterdam
-                              Vu University Amsterdam
+    Copyright (c)  2006-2025, University of Amsterdam
+                              VU University Amsterdam
+                              SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -36,6 +37,7 @@
 :- module(prolog_source,
           [ prolog_read_source_term/4,  % +Stream, -Term, -Expanded, +Options
             read_source_term_at_location/3, %Stream, -Term, +Options
+            prolog_file_directives/3,   % +File, -Directives, +Options
             prolog_open_source/2,       % +Source, -Stream
             prolog_close_source/1,      % +Stream
             prolog_canonical_source/2,  % +Spec, -Id
@@ -45,15 +47,16 @@
             file_name_on_path/2,        % +File, -PathSpec
             file_alias_path/2,          % ?Alias, ?Dir
             path_segments_atom/2,       % ?Segments, ?Atom
-            directory_source_files/3    % +Dir, -Files, +Options
+            directory_source_files/3,   % +Dir, -Files, +Options
+            valid_term_position/2       % +Term, +TermPos
           ]).
-:- autoload(library(apply),[maplist/2]).
-:- autoload(library(debug),[debug/3,assertion/1]).
-:- autoload(library(error),[domain_error/2]).
-:- autoload(library(lists),[member/2,last/2,select/3,append/3]).
-:- autoload(library(operators),
-	    [push_op/3,push_operators/1,pop_operators/0]).
-:- autoload(library(option),[select_option/4,option/3,option/2]).
+:- use_module(library(debug), [debug/3, assertion/1]).
+:- autoload(library(apply), [maplist/2, maplist/3, foldl/4]).
+:- autoload(library(error), [domain_error/2, is_of_type/2]).
+:- autoload(library(lists), [member/2, last/2, select/3, append/3, selectchk/3]).
+:- autoload(library(operators), [push_op/3, push_operators/1, pop_operators/0]).
+:- autoload(library(option), [select_option/4, option/3, option/2]).
+:- autoload(library(modules),[in_temporary_module/3]).
 
 
 /** <module> Examine Prolog source-files
@@ -90,6 +93,7 @@ users of the library are:
     prolog:xref_open_source/2,       % +SourceId, -Stream
     prolog:xref_close_source/2,      % +SourceId, -Stream
     prolog:alternate_syntax/4,       % Syntax, +Module, -Setup, -Restore
+    prolog:xref_update_syntax/2,     % +Directive, +Module
     prolog:quasi_quotation_syntax/2. % Syntax, Library
 
 
@@ -224,24 +228,26 @@ requires_library((:- draw_begin_shape(_,_,_,_)),   library(pcedraw)).
 requires_library((:- use_module(library(pce))),    library(pce)).
 requires_library((:- pce_begin_class(_,_)),        library(pce)).
 requires_library((:- pce_begin_class(_,_,_)),      library(pce)).
+requires_library((:- html_meta(_)),                library(http/html_decl)).
 
 %!  update_state(+Term, +Expanded, +Module) is det.
 %
-%   Update operators and style-check options from the expanded term.
+%   Update operators and style-check options from Term or Expanded.
 
 :- multifile
     pce_expansion:push_compile_operators/1,
     pce_expansion:pop_compile_operators/0.
 
-update_state(Raw, _, _) :-
-    Raw == (:- pce_end_class),
-    !,
+update_state((:- pce_end_class), _, _) =>
     ignore(pce_expansion:pop_compile_operators).
-update_state(Raw, _, SM) :-
-    subsumes_term((:- pce_extend_class(_)), Raw),
-    !,
+update_state((:- pce_extend_class(_)), _, SM) =>
     pce_expansion:push_compile_operators(SM).
-update_state(_Raw, Expanded, M) :-
+update_state(Raw, _, Module),
+    catch(prolog:xref_update_syntax(Raw, Module),
+          error(_,_),
+          fail) =>
+    true.
+update_state(_Raw, Expanded, M) =>
     update_state(Expanded, M).
 
 update_state(Var, _) :-
@@ -260,8 +266,16 @@ update_state((:- Directive), M) :-
 update_state((?- Directive), M) :-
     !,
     update_state((:- Directive), M).
+update_state(html_write:html_meta_head(Head,Module,Meta), _M) :-
+    (   html_write:html_meta_head(Head,Module,Meta)
+    ->  true
+    ;   assertz(html_write:html_meta_head(Head,Module,Meta))
+    ).
 update_state(_, _).
 
+update_directive(Directive, Module) :-
+    prolog:xref_update_syntax((:- Directive), Module),
+    !.
 update_directive(module(Module, Public), _) :-
     atom(Module),
     is_list(Public),
@@ -353,21 +367,29 @@ load_quasi_quotation_syntax(SM:Path, Syntax) :-
     !,
     use_module(SM:Path, [Syntax/4]).
 
-%!  module_decl(+FileSpec, -Path, -Decl) is semidet.
+%!  module_decl(+FileSpec, -Source, -Exports) is semidet.
 %
 %   If FileSpec refers to a Prolog  module   file,  unify  Path with the
 %   canonical file path to the file and Decl with the second argument of
 %   the module declaration.
 
-module_decl(Spec, Path, Decl) :-
+module_decl(Spec, Source, Exports) :-
     absolute_file_name(Spec, Path,
                        [ file_type(prolog),
                          file_errors(fail),
                          access(read)
                        ]),
+    module_decl_(Path, Source, Exports).
+
+module_decl_(Path, Source, Exports) :-
+    file_name_extension(_, qlf, Path),
+    !,
+    '$qlf_module'(Path, Info),
+    _{file:Source, exports:Exports} :< Info.
+module_decl_(Path, Path, Exports) :-
     setup_call_cleanup(
         prolog_open_source(Path, In),
-        read_module_decl(In, Decl),
+        read_module_decl(In, Exports),
         prolog_close_source(In)).
 
 read_module_decl(In, Decl) :-
@@ -593,6 +615,105 @@ prolog:quasi_quotation_syntax(html,       library(http/html_write)).
 prolog:quasi_quotation_syntax(javascript, library(http/js_write)).
 
 
+%!  prolog_file_directives(+File, -Directives, +Options) is det.
+%
+%   True when Directives is a list  of   directives  that  appear in the
+%   source  file  File.  Reading   directives    stops   at   the  first
+%   non-directive term. Processing deals with   expand_term/2 as well as
+%   conditional compilation.  Options processed:
+%
+%     - canonical_source(-Source)
+%       Unify Source with the canonical source identifier as also
+%       used by library(prolog_xref).
+%     - silent(+Boolean)
+%       If `true` (default `false`), do not report syntax errors and
+%       other errors.
+
+prolog_file_directives(File, Directives, Options) :-
+    option(canonical_source(Path), Options, _),
+    prolog_canonical_source(File, Path),
+    in_temporary_module(
+        TempModule,
+        true,
+        read_directives(TempModule, Path, Directives, Options)).
+
+read_directives(TempModule, Path, Directives, Options) :-
+    setup_call_cleanup(
+        read_directives_setup(TempModule, Path, In, State),
+        phrase(read_directives(In, Options, [true]), Directives),
+        read_directives_cleanup(In, State)).
+
+read_directives_setup(TempModule, Path, In, state(OldM, OldXref)) :-
+    prolog_open_source(Path, In),
+    '$set_source_module'(OldM, TempModule),
+    current_prolog_flag(xref, OldXref),
+    set_prolog_flag(xref, true).
+
+read_directives_cleanup(In, state(OldM, OldXref)) :-
+    '$set_source_module'(OldM),
+    set_prolog_flag(xref, OldXref),
+    prolog_close_source(In).
+
+read_directives(In, Options, State) -->
+    {  E = error(_,_),
+       repeat,
+       catch(prolog_read_source_term(In, Term, Expanded,
+                                     [ process_comment(true),
+                                       syntax_errors(error)
+                                     ]),
+             E, report_syntax_error(E, Options))
+    -> nonvar(Term),
+       Term = (:-_)
+    },
+    !,
+    terms(Expanded, State, State1),
+    read_directives(In, Options, State1).
+read_directives(_, _, _) --> [].
+
+report_syntax_error(_, Options) :-
+    option(silent(true), Options),
+    !,
+    fail.
+report_syntax_error(E, _Options) :-
+    print_message(warning, E),
+    fail.
+
+terms(Var, State, State) --> { var(Var) }, !.
+terms([H|T], State0, State) -->
+    !,
+    terms(H, State0, State1),
+    terms(T, State1, State).
+terms((:-if(Cond)), State0, [True|State0]) -->
+    !,
+    { eval_cond(Cond, True) }.
+terms((:-elif(Cond)), [True0|State], [True|State]) -->
+    !,
+    { eval_cond(Cond, True1),
+      elif(True0, True1, True)
+    }.
+terms((:-else), [True0|State], [True|State]) -->
+    !,
+    { negate(True0, True) }.
+terms((:-endif), [_|State], State) -->  !.
+terms(H, State, State) -->
+    (   {State = [true|_]}
+    ->  [H]
+    ;   []
+    ).
+
+eval_cond(Cond, true) :-
+    catch(Cond, error(_,_), fail),
+    !.
+eval_cond(_, false).
+
+elif(true,  _,    else_false) :- !.
+elif(false, true, true) :- !.
+elif(True,  _,    True).
+
+negate(true,       false).
+negate(false,      true).
+negate(else_false, else_false).
+
                  /*******************************
                  *           SOURCES            *
                  *******************************/
@@ -907,6 +1028,146 @@ dir_file_path(Dir, File, Path) :-
     ).
 
 
+%!  valid_term_position(@Term, @TermPos) is semidet.
+%
+%   Check that a Term has an   appropriate  TermPos layout. An incorrect
+%   TermPos results in either failure of this predicate or an error.
+%
+%   If a position in TermPos  is  a   variable,  the  validation  of the
+%   corresponding   part   of   Term   succeeds.    This   matches   the
+%   term_expansion/4 treats "unknown" layout information.   If part of a
+%   TermPos is given, then all its "from"   and "to" information must be
+%   specified; for example,    string_position(X,Y)   is   an  error but
+%   string_position(0,5) succeeds.   The position values are checked for
+%   being plausible -- e.g., string_position(5,0) will fail.
+%
+%   This should always succeed:
+%
+%       read_term(Term, [subterm_positions(TermPos)]),
+%       valid_term_position(Term, TermPos)
+%
+%   @arg Term Any Prolog term including a variable).
+%   @arg TermPos The detailed layout of the term, for example
+%        from using =|read_term(Term, subterm_positions(TermPos)|=.
+%
+%   @error existence_error(matching_rule, Subterm) if a subterm of Term
+%          is inconsistent with the corresponding part of TermPos.
+%
+%   @see read_term/2, read_term/3, term_string/3
+%   @see expand_term/4, term_expansion/4, expand_goal/4, expand_term/4
+%   @see clause_info/4, clause_info/5
+%   @see prolog_clause:unify_clause_hook/5
+
+valid_term_position(Term, TermPos) :-
+    valid_term_position(0, 0x7fffffffffffffff, Term, TermPos).
+
+valid_term_position(OuterFrom, OuterTo, _Term, TermPos),
+        var(TermPos),
+        OuterFrom =< OuterTo => true.
+valid_term_position(OuterFrom, OuterTo, Var, From-To),
+        var(Var),
+        valid_term_position_from_to(OuterFrom, OuterTo, From, To) => true.
+valid_term_position(OuterFrom, OuterTo, Atom, From-To),
+        atom(Atom),
+        valid_term_position_from_to(OuterFrom, OuterTo, From, To) => true.
+valid_term_position(OuterFrom, OuterTo, Number, From-To),
+        number(Number),
+        valid_term_position_from_to(OuterFrom, OuterTo, From, To) => true.
+valid_term_position(OuterFrom, OuterTo, [], From-To),
+        valid_term_position_from_to(OuterFrom, OuterTo, From, To) => true.
+valid_term_position(OuterFrom, OuterTo, String, string_position(From,To)),
+        (   string(String)
+        ->  true
+        ;   is_of_type(codes, String)
+        ->  true
+        ;   is_of_type(chars, String)
+        ->  true
+        ;   atom(String)
+        ),
+        valid_term_position_from_to(OuterFrom, OuterTo, From, To) => true.
+valid_term_position(OuterFrom, OuterTo, {Arg},
+                    brace_term_position(From,To,ArgPos)),
+        valid_term_position_from_to(OuterFrom, OuterTo, From, To) =>
+    valid_term_position(From, To, Arg, ArgPos).
+valid_term_position(OuterFrom, OuterTo, [Hd|Tl],
+                    list_position(From,To,ElemsPos,none)),
+        valid_term_position_from_to(OuterFrom, OuterTo, From, To) =>
+    term_position_list_tail([Hd|Tl], _HdPart, []),
+    maplist(valid_term_position, [Hd|Tl], ElemsPos).
+valid_term_position(OuterFrom, OuterTo, [Hd|Tl],
+                    list_position(From, To, ElemsPos, TailPos)),
+        valid_term_position_from_to(OuterFrom, OuterTo, From, To) =>
+    term_position_list_tail([Hd|Tl], HdPart, Tail),
+    maplist(valid_term_position(From,To), HdPart, ElemsPos),
+    valid_term_position(Tail, TailPos).
+valid_term_position(OuterFrom, OuterTo, Term,
+                    term_position(From,To, FFrom,FTo,SubPos)),
+        valid_term_position_from_to(OuterFrom, OuterTo, From, To) =>
+    compound_name_arguments(Term, Name, Arguments),
+    valid_term_position(Name, FFrom-FTo),
+    maplist(valid_term_position(From,To), Arguments, SubPos).
+valid_term_position(OuterFrom, OuterTo, Dict,
+                    dict_position(From,To,TagFrom,TagTo,KeyValuePosList)),
+        valid_term_position_from_to(OuterFrom, OuterTo, From, To) =>
+    dict_pairs(Dict, Tag, Pairs),
+    valid_term_position(Tag, TagFrom-TagTo),
+    foldl(valid_term_position_dict(From,To), Pairs, KeyValuePosList, []).
+% key_value_position(From, To, SepFrom, SepTo, Key, KeyPos, ValuePos)
+% is handled in valid_term_position_dict.
+valid_term_position(OuterFrom, OuterTo, Term,
+                    parentheses_term_position(From,To,ContentPos)),
+        valid_term_position_from_to(OuterFrom, OuterTo, From, To) =>
+    valid_term_position(From, To, Term, ContentPos).
+valid_term_position(OuterFrom, OuterTo, _Term,
+                    quasi_quotation_position(From,To,
+                                             SyntaxTerm,SyntaxPos,_ContentPos)),
+        valid_term_position_from_to(OuterFrom, OuterTo, From, To) =>
+    valid_term_position(From, To, SyntaxTerm, SyntaxPos).
+
+valid_term_position_from_to(OuterFrom, OuterTo, From, To) :-
+    integer(OuterFrom),
+    integer(OuterTo),
+    integer(From),
+    integer(To),
+    OuterFrom =< OuterTo,
+    From =< To,
+    OuterFrom =< From,
+    To =< OuterTo.
+
+:- det(valid_term_position_dict/5).
+valid_term_position_dict(OuterFrom, OuterTo, Key-Value,
+                         KeyValuePosList0, KeyValuePosList1) :-
+    selectchk(key_value_position(From,To,SepFrom,SepTo,Key,KeyPos,ValuePos),
+              KeyValuePosList0, KeyValuePosList1),
+    valid_term_position_from_to(OuterFrom, OuterTo, From, To),
+    valid_term_position_from_to(OuterFrom, OuterTo, SepFrom, SepTo),
+    SepFrom >= OuterFrom,
+    valid_term_position(From, SepFrom, Key, KeyPos),
+    valid_term_position(SepTo, To, Value, ValuePos).
+
+%!  term_position_list_tail(@List, -HdPart, -Tail) is det.
+%
+%   Similar to append(HdPart, [Tail], List) for   proper lists, but also
+%   works for inproper lists, in which  case   it  unifies Tail with the
+%   tail of the partial list. HdPart is always a proper list:
+%
+%   ```
+%   ?- prolog_source:term_position_list_tail([a,b,c], Hd, Tl).
+%   Hd = [a, b, c],
+%   Tl = [].
+%   ?- prolog_source:term_position_list_tail([a,b|X], Hd, Tl).
+%   X = Tl,
+%   Hd = [a, b].
+%   ```
+
+:- det(term_position_list_tail/3).
+term_position_list_tail([X|Xs], HdPart, Tail) =>
+    HdPart = [X|HdPart2],
+    term_position_list_tail(Xs, HdPart2, Tail).
+term_position_list_tail(Tail0, HdPart, Tail) =>
+    HdPart = [],
+    Tail0 = Tail.
+
 
                  /*******************************
                  *           MESSAGES           *
