@@ -3,9 +3,8 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2007-2024, University of Amsterdam
+    Copyright (c)  2007-2015, University of Amsterdam
                               VU University Amsterdam
-                              SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -188,7 +187,7 @@ process_items(PI, Mode0, Mode, Options) -->
     latex_tokens_for_predicates(PI, Options).
 process_items(FileSpec, Mode0, Mode, Options) -->
     {   (   absolute_file_name(FileSpec,
-                               [ file_type(prolog),
+                               [ file_type(source),
                                  access(read),
                                  file_errors(fail)
                                ],
@@ -650,8 +649,8 @@ summed_string_len([_|T], L0, L) :-
 %   Emit a LaTeX section,  keeping  track   of  the  desired highest
 %   section level.
 %
-%   @param Level    Desired level, relative to the base-level.  Must
-%                   be a non-negative integer.
+%   @arg Level    Desired level, relative to the base-level.  Must
+%                 be a non-negative integer.
 
 latex_section(Level, Attrs, Content) -->
     { current_options(Options),
@@ -873,7 +872,7 @@ argument(arg(Name,Descr)) -->
 file_header(File, Options) -->
     { memberchk(file(Title, Comment), Options),
       !,
-      file_synopsis(File, Synopsis)
+      file_synopsis(File, Synopsis, Options)
     },
     file_title([Synopsis, ': ', Title], File, Options),
     { is_structured_comment(Comment, Prefixes),
@@ -886,7 +885,7 @@ file_header(File, Options) -->
     latex(DOM),
     latex(cmd(vspace('0.7cm'))).
 file_header(File, Options) -->
-    { file_synopsis(File, Synopsis)
+    { file_synopsis(File, Synopsis, Options)
     },
     file_title([Synopsis], File, Options).
 
@@ -896,7 +895,10 @@ tags_to_front(DOM0, DOM) :-
     DOM = [\tags(Tags)|Content].
 tags_to_front(DOM, DOM).
 
-file_synopsis(File, Synopsis) :-
+file_synopsis(_File, Synopsis, Options) :-
+    option(file_synopsis(Synopsis), Options),
+    !.
+file_synopsis(File, Synopsis, _) :-
     file_name_on_path(File, Term),
     unquote_filespec(Term, Unquoted),
     format(atom(Synopsis), '~w', [Unquoted]).
@@ -911,8 +913,11 @@ file_title(Title, File, Options) -->
       Section =.. [Level,Title],
       file_base_name(File, BaseExt),
       file_name_extension(Base, _, BaseExt),
-      delete_unsafe_label_chars(Base, SafeBase),
-      atom_concat('sec:', SafeBase, Label)
+      (   option(label(Seclabel), Options)
+      ->  true
+      ;   delete_unsafe_label_chars(Base, Seclabel)
+      ),
+      atom_concat('sec:', Seclabel, Label)
     },
     latex(cmd(Section)),
     latex(cmd(label(Label))).
@@ -1645,6 +1650,11 @@ nl(Out, N) :-
     forall(between(1, N, _), nl(Out)).
 
 
+%!  print_char(+Char, +Out) is det.
+%
+%   Write Char in LaTeX format to Out. This escapes characters for LaTeX
+%   where necessary.
+
 print_char('<', Out) :- !, write(Out, '$<$').
 print_char('>', Out) :- !, write(Out, '$>$').
 print_char('{', Out) :- !, write(Out, '\\{').
@@ -1657,9 +1667,39 @@ print_char('~', Out) :- !, write(Out, '\\Stilde{}').
 print_char('\\',Out) :- !, write(Out, '\\bsl{}').
 print_char('^', Out) :- !, write(Out, '\\Shat{}').
 print_char('|', Out) :- !, write(Out, '\\Sbar{}').
-print_char('ö', Out) :- !, write(Out, '\\"o').
+print_char(C,   Out) :- decompose_char(C, Out), !.
 print_char(C,   Out) :- put_char(Out, C).
 
+%!  decompose_char(+Char) is semidet.
+%
+%   Deal with diacritics.  Relies  on   Unicode  decomposition,  where a
+%   character with diacritics becomes the plain character, followed by a
+%   composing diacritics mark.
+
+:- if(exists_source(library(unicode))).
+:- use_module(library(unicode)).
+decompose_char(Char, Out) :-
+    char_code(Char, Code),
+    Code > 128,
+    unicode_map(Char, Decomposed, [decompose]),
+    atom_codes(Decomposed, [C,D]),
+    diacritic_cmd(D, Cmd),
+    format(Out, '\\~w~c', [Cmd, C]).
+:- else.
+decompose_char(_,_) :-
+    fail.
+:- endif.
+
+diacritic_cmd(768, '`').
+diacritic_cmd(769, '\'').
+diacritic_cmd(770, '~').
+diacritic_cmd(771, '=').
+diacritic_cmd(774, 'v').
+diacritic_cmd(775, '.').
+diacritic_cmd(776, '"').
+diacritic_cmd(785, 'u').
+diacritic_cmd(807, 'c').
+diacritic_cmd(808, 'k').
 
 %!  identifier(+Atom) is semidet.
 %
