@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2017-2021, VU University Amsterdam
+    Copyright (c)  2017-2025, VU University Amsterdam
 			      CWI, Amsterdam
 			      SWI-Prolog Solutions b.v
     All rights reserved.
@@ -62,7 +62,7 @@ typedef struct trie_children_key
 
 typedef struct trie_children_hashed
 { tn_node_type	type;			/* TN_HASHED */
-  Table		table;			/* Key --> child map */
+  TableWP	table;			/* Key --> child map */
   unsigned	var_mask;		/* Variables in this place */
   trie_children_key *old_single;	/* Old single node */
 } trie_children_hashed;
@@ -80,11 +80,10 @@ typedef union trie_children
 #define TN_IDG_DELETED			0x0008	/* IDG pre-evaluation */
 #define TN_IDG_ADDED			0x0010	/* IDG recovery */
 #define TN_IDG_UNCONDITIONAL		0x0020	/* IDG: previous cond state */
-#define TN_IDG_SAVED_UNCONDITIONAL	0x0040	/* IDG recovery */
-#define TN_IDG_AS_LAST			0x0080	/* IDG: answer subsumption node */
+#define TN_IDG_AS_LAST			0x0040	/* IDG: answer subsumption node */
 #define TN_IDG_MASK \
 	(TN_IDG_DELETED|TN_IDG_ADDED| \
-	 TN_IDG_UNCONDITIONAL|TN_IDG_SAVED_UNCONDITIONAL|TN_IDG_AS_LAST)
+	 TN_IDG_UNCONDITIONAL|TN_IDG_AS_LAST)
 
 typedef struct trie_node
 { word			value;
@@ -114,7 +113,7 @@ typedef struct trie
 #ifdef O_PLMT
   int			tid;		/* thread id doing completion or re-evaluation */
 #endif
-  trie_node	        root;		/* the root node */
+  trie_node		root;		/* the root node */
   indirect_table       *indirects;	/* indirect values */
   void		      (*release_node)(struct trie *, trie_node *);
   alloc_pool	       *alloc_pool;	/* Node allocation pool */
@@ -161,54 +160,77 @@ typedef struct size_abstract
 #define TRIE_LOOKUP_CONTAINS_ATTVAR	-10
 #define TRIE_LOOKUP_CYCLIC		-11
 
-COMMON(void)	initTries(void);
-COMMON(trie *)	trie_create(alloc_pool *pool);
-COMMON(void)	trie_destroy(trie *trie);
-COMMON(void)	trie_empty(trie *trie);
-COMMON(void)	trie_clean(trie *trie);
-COMMON(void)	trie_delete(trie *trie, trie_node *node, int prune);
-COMMON(void)	prune_node(trie *trie, trie_node *n);
-COMMON(void)	prune_trie(trie *trie, trie_node *root,
-			   void (*free)(trie_node *node, void *ctx), void *ctx);
-COMMON(trie *)	get_trie_from_node(trie_node *node);
-COMMON(int)	is_ground_trie_node(trie_node *node);
-COMMON(int)	is_leaf_trie_node(trie_node *n);
-COMMON(int)	get_trie(term_t t, trie **tp);
-COMMON(int)	get_trie_noex(term_t t, trie **tp);
-COMMON(int)	unify_trie_term(trie_node *node, trie_node **parent,
-				term_t term ARG_LD);
-COMMON(int)	trie_lookup_abstract(trie *trie,
-				     trie_node *root, trie_node **nodep, Word k,
-				     int add, size_abstract *abstract,
-				     TmpBuffer vars ARG_LD);
-COMMON(int)	trie_error(int rc, term_t culprit);
-COMMON(int)	trie_trie_error(int rc, trie *trie);
-COMMON(atom_t)	trie_symbol(trie *trie);
-COMMON(trie *)	symbol_trie(atom_t symbol);
-COMMON(int)	put_trie_value(term_t t, trie_node *node ARG_LD);
-COMMON(int)	set_trie_value(trie *trie, trie_node *node, term_t value ARG_LD);
-COMMON(int)	set_trie_value_word(trie *trie, trie_node *node, word val);
-COMMON(foreign_t) trie_gen_raw(
-		      trie *trie, trie_node *root,
-		      term_t Key, term_t Value,
-		      term_t Data,
-		      int (*unify_data)(term_t, trie_node*, void* ARG_LD),
-		      void *ctx, control_t PL__ctx);
-COMMON(foreign_t) clear_trie_gen_state(void *ctx);
-COMMON(foreign_t) trie_gen(term_t Trie, term_t Root, term_t Key, term_t Value,
-			   term_t Data,
-			   int (*unify_data)(term_t, trie_node*, void* ARG_LD),
-			   void *ctx, control_t PL__ctx);
-COMMON(void *)	map_trie_node(trie_node *n,
-			      void* (*map)(trie_node *n, void *ctx), void *ctx);
-COMMON(atom_t)	compile_trie(Definition def, trie *trie ARG_LD);
-COMMON(void)	trie_discard_clause(trie *trie);
+#if USE_LD_MACROS
+#define	unify_trie_term(node, parent, term)					LDFUNC(unify_trie_term, node, parent, term)
+#define	trie_lookup_abstract(trie, root, nodep, k, add, abstract, vars)		LDFUNC(trie_lookup_abstract, trie, root, nodep, k, add, abstract, vars)
+#define	put_trie_value(t, node)							LDFUNC(put_trie_value, t, node)
+#define	set_trie_value(trie, node, value)					LDFUNC(set_trie_value, trie, node, value)
+#define	compile_trie(def, trie)							LDFUNC(compile_trie, def, trie)
+#endif /*USE_LD_MACROS*/
 
+#define LDFUNC_DECLARATIONS
+
+void	initTries(void);
+trie *	trie_create(alloc_pool *pool);
+int	release_trie_ref(atom_t aref);
+void	trie_empty(trie *trie);
+void	trie_clean(trie *trie);
+void	trie_delete(trie *trie, trie_node *node, int prune);
+void	prune_node(trie *trie, trie_node *n);
+void	prune_trie(trie *trie, trie_node *root,
+		   void (*free)(trie_node *node, void *ctx), void *ctx);
+trie *	get_trie_from_node(trie_node *node);
+bool	is_ground_trie_node(trie_node *node);
+bool	is_leaf_trie_node(trie_node *n);
+bool	get_trie(term_t t, trie **tp);
+bool	get_trie_noex(term_t t, trie **tp);
+bool	unify_trie_term(trie_node *node, trie_node **parent,
+			term_t term);
+int	trie_lookup_abstract(trie *trie,
+			     trie_node *root, trie_node **nodep, Word k,
+			     bool add, size_abstract *abstract,
+			     TmpBuffer vars);
+bool	trie_error(int rc, term_t culprit);
+bool	trie_trie_error(int rc, trie *trie);
+atom_t	trie_symbol(trie *trie);
+trie *	symbol_trie(atom_t symbol);
+bool	put_trie_value(term_t t, trie_node *node);
+bool	set_trie_value(trie *trie, trie_node *node, term_t value);
+bool	set_trie_value_word(trie *trie, trie_node *node, word val);
+foreign_t trie_gen_raw(
+	      trie *trie, trie_node *root,
+	      term_t Key, term_t Value,
+	      term_t Data,
+	      int LDFUNCP (*unify_data)(term_t, trie_node*, void*),
+	      void *ctx, control_t PL__ctx);
+foreign_t clear_trie_gen_state(void *ctx);
+foreign_t trie_gen(term_t Trie, term_t Root, term_t Key, term_t Value,
+		   term_t Data,
+		   int LDFUNCP (*unify_data)(term_t, trie_node*, void*),
+		   void *ctx, control_t PL__ctx);
+void *	map_trie_node(trie_node *n,
+		      void* (*map)(trie_node *n, void *ctx), void *ctx);
+atom_t	compile_trie(Definition def, trie *trie);
+void	trie_discard_clause(trie *trie);
+
+#undef LDFUNC_DECLARATIONS
+
+#ifndef NO_TRIE_GEN_HELPERS
+/* The common case is passing in the name of an LDFUNC, so this adds the LDFUNC_REF()
+ * to get proper mangling. To use this without the mangling either define the
+ * NO_TRIE_GEN_HELPERS macro (like pl-trie.c does) or put (trie_gen_raw) in parentheses. */
+#define trie_gen_raw(trie, root, Key, Value, Data, unify_func, ctx, PL__ctx) \
+	trie_gen_raw(trie, root, Key, Value, Data, LDFUNC_REF(unify_func), ctx, PL__ctx)
+#define trie_gen(Trie, Root, Key, Value, Data, unify_func, ctx, PL__ctx) \
+	trie_gen(Trie, Root, Key, Value, Data, LDFUNC_REF(unify_func), ctx, PL__ctx)
+#endif
+
+#define trie_lookup(trie, node, nodep, k, add, vars) LDFUNC(trie_lookup, trie, node, nodep, k, add, vars)
 static inline int
-trie_lookup(trie *trie, trie_node *node, trie_node **nodep,
-	    Word k, int add, TmpBuffer vars ARG_LD)
+trie_lookup(DECL_LD trie *trie, trie_node *node, trie_node **nodep,
+	    Word k, int add, TmpBuffer vars)
 { return trie_lookup_abstract(trie, node, nodep, k, add,
-			      NULL, vars PASS_LD);
+			      NULL, vars);
 }
 
 #endif /*_PL_TRIE_H*/
